#ifndef __OPENGLRENDER_H__
#define __OPENGLRENDER_H__

#include "render.h"
#include "DirectDraw.h"
#include "Direct3D.h"
#include "View.h"
#include "Texture.h"
#include "Poly.h"

class OpenGLRender: public Render
{
private:
    int init;
    HWND hWnd;
    Poly farPoly;
    int stencilValue;
    HDC dc;
    PolyList *extraPolyList;
    Vector viewVect;
    struct FontListElem
    {
        int id;
        int width,height;
        FontListElem *next;
    };
    FontListElem *firstFont;
    FontListElem *lastFont;
    FontListElem *curFont;
    int polyDrawn,polyTotal,polyReflect;
    bool renderEnabled;
    bool sceneEnabled;
    HGLRC hContext;
    float x,y,z,xa,ya,za;
    BYTE convBuf[65536*4];
    int fontDisplayList;
    int translateMode;

    void RenderPolyListNonBSP(PolyList &list);
    void UpdateViewMatrix();
public:
    OpenGLRender(HINSTANCE hInst,HWND hWnd,int scrnX,int scrnY,int depth);
    ~OpenGLRender();
    int Initialized();
    void EnumModes(ModeList *modes16,ModeList *modes32);
    void ChangeResolution(int scrnX,int scrnY,int depth);
    void SetRenderWnd(HWND hWnd);
    void BeginRender();
    void RenderPoly(Poly &poly);
    void RenderPolyList(PolyList &list);
    void EndRender();
    void BeginScene();
    void EndScene();
    void SetXPos(float x);
    void SetYPos(float y);
    void SetZPos(float z);
    void AddXPos(float x);
    void AddYPos(float y);
    void AddZPos(float z);
    void SetXRotate(float x);
    void SetYRotate(float y);
    void SetZRotate(float z);
    void AddXRotate(float x);
    void AddYRotate(float y);
    void AddZRotate(float z);
    void GoForward(float d);
    void GoSide(float d);
    float GetXPos();
    float GetYPos();
    float GetZPos();
    float GetXRotate();
    float GetYRotate();
    float GetZRotate();
    Texture* LoadTextureBMP(char *filename);
    Texture* LoadAlphaTextureBMP(char *filename,char *alphaname);
    void CreateMemoryTexture(Texture* tex);
    void UpdateMemoryTexture(Texture* tex);
    void ReloadTexture(Texture* tex);
    void FreeTexture(Texture* tex);
    void UseOriginView();
    void UseNormalView();
    void StartText();
    void EndText();
    void RenderText(int x,int y,char *text,unsigned long color);
    void* AddFont(HFONT font);
    void UseFont(void* font);
    void GetStats(RenderStats *stat);
    void BeginBitmaps();
    void RenderBitmap(int x,int y,Art *art,int texIndex,unsigned long color);
    void RenderBitmap(int x,int y,Art *art,int texIndex,int palNum,unsigned long color);
    void RenderBitmap(int x,int y,Texture *tex,int w,int h,unsigned long color);
    void RenderBitmapYFlipped(int x,int y,Art *art,int texIndex,unsigned long color);
    void RenderScaledBitmap(int x,int y,int w,int h,Art *art,int texIndex,unsigned long color);
    void RenderScaledBitmap(int x,int y,int w,int h,Art *art,int texIndex,int palNum,unsigned long color);
    void EndBitmaps();
};

#endif
