#include "poly.h"
#include <windows.h>
#include <math.h>

#define SQ(x) ((x)*(x))

Plane PlaneFromPoly(Poly& poly)
{
    Vector normal=VectorCrossProduct(Vector(poly.v[1].x-poly.v[0].x,
        poly.v[1].y-poly.v[0].y,poly.v[1].z-poly.v[0].z),Vector(
        poly.v[2].x-poly.v[0].x,poly.v[2].y-poly.v[0].y,poly.v[2].z-
        poly.v[0].z));
    return Plane(normal.x,normal.y,normal.z,-(normal.x*
        poly.v[0].x+normal.y*poly.v[0].y+normal.z*poly.v[0].z));
}

Vector PolyNormal(Poly& poly)
{
    Vector normal=VectorCrossProduct(Vector(poly.v[1].x-poly.v[0].x,
        poly.v[1].y-poly.v[0].y,poly.v[1].z-poly.v[0].z),Vector(
        poly.v[2].x-poly.v[0].x,poly.v[2].y-poly.v[0].y,poly.v[2].z-
        poly.v[0].z));
    if (poly.flags&PF_REVERSE)
        normal=Vector(-normal.x,-normal.y,-normal.z);
    return normal;
}

int IsBackFace(Poly& poly,Vector& pos)
{
    if (poly.flags&PF_TWOSIDED) return 0;
    Vector normal=VectorCrossProduct(Vector(poly.v[1].x-poly.v[0].x,
        poly.v[1].y-poly.v[0].y,poly.v[1].z-poly.v[0].z),Vector(
        poly.v[2].x-poly.v[0].x,poly.v[2].y-poly.v[0].y,poly.v[2].z-
        poly.v[0].z));
    int result=normal.x*pos.x+normal.y*pos.y+normal.z*pos.z-(normal.x*
        poly.v[0].x+normal.y*poly.v[0].y+normal.z*poly.v[0].z);
    return (poly.flags&PF_REVERSE)?(result>0):(result<0);
}

PolyListElem* AddPolyToList(PolyList &list,Poly &poly)
{
    poly.CalcMinRange();
    if (list.first==NULL)
    {
        list.first=list.last=new PolyListElem;
        list.first->poly=*(new Poly(poly));
        return list.first;
    }
    list.last->next=new PolyListElem;
    list.last->next->poly=*(new Poly(poly));
    list.last=list.last->next;
    return list.last;
}

void FreeList(PolyList &list)
{
    PolyListElem *ptr=list.first;
    while (ptr!=NULL)
    {
        PolyListElem *tmp=ptr->next;
        delete ptr;
        ptr=tmp;
    }
    list.first=list.last=NULL;
}

Vector VectorCrossProduct(Vector& u,Vector& v)
{
    return Vector(u.y*v.z-u.z*v.y,-(u.x*v.z-u.z*v.x),
        u.x*v.y-u.y*v.x);
}

int IsPointInPolyXY(Vector& pt,Poly &poly,float minDist)
{
    int i,j,c=0;
    for (i=0,j=poly.nVert-1;i<poly.nVert;j=i++)
    {
        if ((((poly.v[i].y<=pt.y)&&(pt.y<poly.v[j].y))||
            ((poly.v[j].y<=pt.y)&&(pt.y<poly.v[i].y)))&&
            (pt.x<(poly.v[j].x-poly.v[i].x)*(pt.y-
            poly.v[i].y)/(poly.v[j].y-poly.v[i].y)+
            poly.v[i].x))
            c=!c;
    }
    if (c) return 1;
    if (minDist==0) return 0;
    for (i=0,j=poly.nVert-1;i<poly.nVert;j=i++)
    {
        Vector edge=Vector(poly.v[j].x-poly.v[i].x,poly.v[j].y-
            poly.v[i].y,poly.v[j].z-poly.v[i].z);
        Vector ptv=Vector(pt.x-poly.v[i].x,pt.y-poly.v[i].y,
            pt.z-poly.v[i].z);
        float scale=(edge.x*ptv.x+edge.y*ptv.y+edge.z*ptv.z)/
            (edge.x*edge.x+edge.y*edge.y+edge.z*edge.z);
        Vector proj=Vector(edge.x*scale,edge.y*scale,
            edge.z*scale);
        float distsq=(ptv.x-proj.x)*(ptv.x-proj.x)+
            (ptv.y-proj.y)*(ptv.y-proj.y)+(ptv.z-proj.z)*
            (ptv.z-proj.z);
        if (distsq<(minDist-0.00001))
            return 1;
    }
    return 0;
}

int IsPointInPolyXZ(Vector& pt,Poly &poly,float minDist)
{
    int i,j,c=0;
    for (i=0,j=poly.nVert-1;i<poly.nVert;j=i++)
    {
        if ((((poly.v[i].z<=pt.z)&&(pt.z<poly.v[j].z))||
            ((poly.v[j].z<=pt.z)&&(pt.z<poly.v[i].z)))&&
            (pt.x<(poly.v[j].x-poly.v[i].x)*(pt.z-
            poly.v[i].z)/(poly.v[j].z-poly.v[i].z)+
            poly.v[i].x))
            c=!c;
    }
    if (c) return 1;
    if (minDist==0) return 0;
    for (i=0,j=poly.nVert-1;i<poly.nVert;j=i++)
    {
        Vector edge=Vector(poly.v[j].x-poly.v[i].x,poly.v[j].y-
            poly.v[i].y,poly.v[j].z-poly.v[i].z);
        Vector ptv=Vector(pt.x-poly.v[i].x,pt.y-poly.v[i].y,
            pt.z-poly.v[i].z);
        float scale=(edge.x*ptv.x+edge.y*ptv.y+edge.z*ptv.z)/
            (edge.x*edge.x+edge.y*edge.y+edge.z*edge.z);
        Vector proj=Vector(edge.x*scale,edge.y*scale,
            edge.z*scale);
        float distsq=(ptv.x-proj.x)*(ptv.x-proj.x)+
            (ptv.y-proj.y)*(ptv.y-proj.y)+(ptv.z-proj.z)*
            (ptv.z-proj.z);
        if (distsq<(minDist-0.00001))
            return 1;
    }
    return 0;
}

int IsPointInPolyYZ(Vector& pt,Poly &poly,float minDist)
{
    int i,j,c=0;
    for (i=0,j=poly.nVert-1;i<poly.nVert;j=i++)
    {
        if ((((poly.v[i].z<=pt.z)&&(pt.z<poly.v[j].z))||
            ((poly.v[j].z<=pt.z)&&(pt.z<poly.v[i].z)))&&
            (pt.y<(poly.v[j].y-poly.v[i].y)*(pt.z-
            poly.v[i].z)/(poly.v[j].z-poly.v[i].z)+
            poly.v[i].y))
            c=!c;
    }
    if (c) return 1;
    if (minDist==0) return 0;
    for (i=0,j=poly.nVert-1;i<poly.nVert;j=i++)
    {
        Vector edge=Vector(poly.v[j].x-poly.v[i].x,poly.v[j].y-
            poly.v[i].y,poly.v[j].z-poly.v[i].z);
        Vector ptv=Vector(pt.x-poly.v[i].x,pt.y-poly.v[i].y,
            pt.z-poly.v[i].z);
        float scale=(edge.x*ptv.x+edge.y*ptv.y+edge.z*ptv.z)/
            (edge.x*edge.x+edge.y*edge.y+edge.z*edge.z);
        Vector proj=Vector(edge.x*scale,edge.y*scale,
            edge.z*scale);
        float distsq=(ptv.x-proj.x)*(ptv.x-proj.x)+
            (ptv.y-proj.y)*(ptv.y-proj.y)+(ptv.z-proj.z)*
            (ptv.z-proj.z);
        if (distsq<(minDist-0.00001))
            return 1;
    }
    return 0;
}

int IsPointInPoly(Vector& pt,Poly &poly,float minDist)
{
    switch(poly.minRange)
    {
        case 0: return IsPointInPolyYZ(pt,poly,minDist);
        case 1: return IsPointInPolyXZ(pt,poly,minDist);
        case 2: return IsPointInPolyXY(pt,poly,minDist);
    }
    return 0;
}

